/* 
 * This file is part of the Hyper-Sphere Uniform Point Generator.
 *
 * Hyper-Sphere Uniform Point Generator by Luis M. S. Russo and
 * Alexandre P. Francisco / KDBIO / INESC-ID is licensed under a Creative
 * Commons Attribution 3.0 Unported License.  Permissions beyond the
 * scope of this license may be available at qhv@kdbio.inesc-id.pt
 *
 * Any published media that is related with to use of the distributed 
 * software, or derived software, must contain a reference to "Extending 
 * quick hypervolume. Luís M. S. Russo, Alexandre P. Francisco: 
 * J. Heuristics 22(3): 245-271 (2016)".
 *  
 * Permission to use, copy, modify, and/or distribute this software for 
 * any purpose with or without fee is hereby granted, provided that the 
 * above copyright notice and this permission notice appear in all 
 * copies. 
 *  
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL 
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE 
 * AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR 
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER 
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR 
 * PERFORMANCE OF THIS SOFTWARE. 
 * 
 */

/* Note: Based on the geometrical distribution. */

#define _XOPEN_SOURCE
#define _XOPEN_SOURCE_EXTENDED
#include <stdio.h>
#include <stdlib.h>
#include <math.h>


#define DIM 40 /* Maximal number of dimensions */

double P[DIM];

double Norm(double* V, int d)
{
  double r;

  r = 0;
  while(d > 0)
    {
      d--;
      r += V[d]*V[d];
    }

  return sqrt(r);
}

void Scale(double* V, int d, double s)
{
  while(d > 0)
    {
      d--;
      V[d] *= s;
    }
}


/* Choosing a point from a normal distribution with average 0 */
void chooseRandom(double* V, int d)
{
  /* The distribution is obtained by adding S uniform values */
#define SAMPS 20
  int j;
  
  while(d > 0)
    {
      d--;
      V[d] = 0;
      j = SAMPS;
      while(j > 0)
        {
          V[d] += drand48() - 0.5;
          j--;
        }
      V[d] = fabs(V[d]);
    }
#undef S
}

void printV(double* V, int d)
{
  while(d > 0)
    {
      d--;
      printf("%.10le ", V[d]);      
    }
  printf("\n");
}

int main(int argc, char** argv)
{
  int dim; /* Dimension */
  int n;   /* Number of points */
  int k;
  int seed;
  double r;
  int i;

  if(argc != 5)
    {
      printf("Usage %s Dimension Number_of_points seed Numbers_of_tests\n", argv[0]);
    }
  else
    {
      sscanf(argv[1], "%d", &dim);      
      sscanf(argv[2], "%d", &n);
      sscanf(argv[3], "%d", &seed);
      sscanf(argv[4], "%d", &k);
     
      srand48(seed);

   while (k--) {

      printf("#\n");

      i = 0;
      while(i < n)
	{
	  chooseRandom(P, dim);
	  r = Norm(P, dim);
	  Scale(P, dim, 1/r);
	  printV(P, dim);
	  i++;
	}
   }
      printf("#\n");
    }
  
  return 0;
}
